/* Grafik-Routinen */

#include "gdi_ks0713.h"

#include "Helvetica_12.c"
#include "Helvetica_Bold_12.c"
#include "Fix5x7_7.c"
#include "ArialBlack_16.c"

extern byte BWSP[];



/************************************************************************/
/* Funktionen und Deklarationen,
   die bei vorigen Versionen
   in Assembler geschrieben waren */

int Get_dY      (const char* Font)    /*  liefert die Zeichenhhe des aktuellen Fonts. */
{ return ((struct Fontheader*) Font)->Height; }

int Get_Ascent  (const char* Font)    /*  liefert die Hhe ber Schreiblinie */
{ return ((struct Fontheader*) Font)->Ascent; }

int Get_Descent (const char* Font)    /*  liefert die Tiefe unter der Schreiblinie */
{ return ((struct Fontheader*) Font)->Descent; }

int Get_dX (const char* Font, char Ch) /* liefert die Zeichenbreite des Zeichens. */
{ struct Fontheader* Fh;
  struct CharEntry*  Ce;
  Fh = (struct Fontheader*) Font;
  Ce = (struct CharEntry*) ( (long) Font + Fontheadersize);
  if (Ch < Fh->ChMin) return 0;
  if (Ch > Fh->ChMax) return 0;
  Ce += (Ch - Fh->ChMin);
  return Ce->dX;
}


/****************************************************************
 Rechteck auf Bildschirm fllen. mode 0..2
 left/top = erste zu fllende Koordinate
 right/bottom = letzte zu fllende Koordinate.
 also CgFillRect(.. 3,3,3,3) fllt genau 1 Pixel auf Position 3/3
 ****************************************************************/

void CgFillRect  (RECT* R, int mode)
{ int  i, left, top, right, bott;
  int  page, mask0, mask1;

  left = R->left;
  top  = R->top;
  right= R->right;
  bott = R->bottom;
  if (left < 0)    left  = 0;
  if (right> 131)  right = 131;
  if (top  < 0)    top   = 0;
  if (bott > 31)   bott  = 31;
  while (top <= bott)
  {  page = (top>>3) * 132;
     mask1 = 1<<(top&7);
     mask0 = ~mask1;

     i = left;
     switch (mode)
     { case 0:             // black
            while (i <= right)
            { BWSP[page+i] |= mask1;
	      ++i;
            }
	    break;
       case 1:             // white
            while (i <= right)
            { BWSP[page+i] &= mask0;
	      ++i;
            }
	    break;
       case 2:             // invert
            while (i <= right)
            { BWSP[page+i] ^= mask1;
	      ++i;
            }
	    break;
     }
     ++top;
  }
}


/****************************************************************
 Pixel auf Koordinate [x,y] mit PenColor schreiben.
 zurckgeliefert wird der bisherige Inhalt des Pixels
 ****************************************************************/
word CgPixel_at (int X, int Y, int mode)
{ word  i;
  int  page, mask0, mask1;

  if (X < 0)   return 0;
  if (X > 131) return 0;
  if (Y < 0)   return 0;
  if (Y > 31)  return 0;
  page = (Y>>3) * 132;
  mask1 = 1<<(Y&7);
  mask0 = ~mask1;
  switch (mode)
  { case 0:             // black
       i = BWSP[page+X];
       BWSP[page+X] |= mask1;
       break;
    case 1:             // white
       i = BWSP[page+X];
       BWSP[page+X] &= mask0;
       break;
    case 2:             // invert
       i = BWSP[page+X];
       BWSP[page+X] ^= mask1;
       break;
  }
  if (i & mask1) return 1;
  return 0;
}


/***************************************************************
 Zeichnet das Textzeichen im aktuellen Font an der
 angegebenen Stelle und liefert die Breite des Zeichens
 ***************************************************************/
int CgCh_at(int X, int Y, char Ch, const char* Font, int mode)
{ struct Fontheader* FHE;
  struct CharEntry*  CHE;
  byte*  Pix;
  long   top, bott;
  dword  w;
  dword  pixw;
  long   dX;

  top   = Y;

/* aus dem Font die Breite, Hhe und Pixel entnehmen */
  FHE = (struct Fontheader*) Font;
  if ((Ch < FHE->ChMin)||(Ch > FHE->ChMax)) Ch = 0;
  else Ch = Ch - FHE->ChMin;
  CHE = (struct CharEntry*) ((long) FHE + Fontheadersize);
  CHE += Ch;
  w   = (CHE->DispHi << 8) | CHE->DispLo;
  Pix = (byte*) ((long) FHE + w);

/* bott setzen */
  bott = top + CHE->dY;
  if (bott>32) bott = 32;

/* die Register sind jetzt:
  bott = zeigt jetzt hinter die unterste zu schreibende Zeile des Zeichens
  Pix  = Zeiger auf zugehrige Pixel im Font
  FHE, CHE haben sich (fast) erledigt
  Nun die ersten Fontpixel und die Maske laden
 */
  pixw = *Pix++;
  w = 1;

/* Y - Zeichen-Zyklus */
  while (top < bott)
  {   /* X - Zeichen-Zyklus */
      dX  = 0;
      while (dX < CHE->dX)   /* Zeichenbreite in Pixeln */
      { if (pixw & w) CgPixel_at (X+dX, top, mode);
	++dX;
        w = (w << 1) & 0xFF;
        if (!w)
        {   pixw = *Pix++;
            w = 1;
	}
      }
      ++top;
  }
  return CHE->dX;
}


/************************************************************************/


int StringWidth (char* S, const char* Font)
{ int i;
  i = 0;
  while (*S) i += Get_dX(Font, *S++);
  return i;
}




void DrawLine (POINT A, POINT B, int mode)
{ int x, y, c;
  int D, HX, HY, M;
  int xInc, yInc;

  x = A.X;
  y = A.Y;
  D = 0;
  HX = B.X - A.X;
  HY = B.Y - A.Y;
  xInc = 1;
  yInc = 1;

  if (HX < 0) { xInc = -1; HX = -HX; };
  if (HY < 0) { yInc = -1; HY = -HY; };
  if (HY <= HX)
  { c = 2 * HX;
    M = 2 * HY;
    loop1:
      CgPixel_at(x,y,mode);
      if (x == B.X) return;
      x = x + xInc;
      D = D + M;
      if (D > HX)
       { y = y + yInc;
         D = D - c;
       }
      goto loop1;
  }
  else
  { c = 2 * HY;
    M = 2 * HX;
    loop2:
      CgPixel_at(x,y,mode);
      if (y == B.Y) return;
      y = y + yInc;
      D = D + M;
      if (D > HY)
       { x = x + xInc;
         D = D - c;
       }
      goto loop2;
  }
}



int CgStr_at (int x, int y, char* P, const char* Font, int mode)
{ int z;
  while (*P)
  { z = CgCh_at(x,y,*P++, Font, mode);
    x = x + z;
  }
  return (x);
}
