/* Definitionen fr das Grafik-Interface */

#ifndef _GDI_INCLUDED
#define _GDI_INCLUDED

#include "stdtypes.h"

#define black  0
#define white  1
#define invert 2


/*  Beschreibung des verwendeten Font-Formates:
 zuerst kommt der Font-Header in folgendem Format: */

#define Fontheadersize 6

struct Fontheader
{ byte Capitals; /* Hoehe der Grossbuchstaben */
  byte Height;   /* Font-Gesamthoehe */
  byte Ascent;   /* Hoehe ueber Null-Linie */
  byte Descent;  /* Tiefe unter Null-Linie */
  char ChMin;    /* kleinstes vorhandenes Zeichen */
  char ChMax;    /* groesstes vorhandenes Zeichen */
};

/* danach kommen unterschiedlich viele Char-Entries.
   eigentlich ist 'Disp' ein 16 bit Offset, aber der
   Font muss bei BigEndian und LittleEndian-CPU's
   gleichermassen funktionieren, deswegen die explizite
   Aufteilung in DispHi und DispLo  */

#define Charentrysize 4

struct CharEntry
{ byte DispHi;   /* Hi vom Displacement ab Fontbeginn */
  byte DispLo;   /* Lo vom Displacement ab Fontbeginn */
  byte dX;       /* Breite in Pixeln */
  byte dY;       /* Hoehe in Pixeln */
};

/* danach kommen die Pixel. Jeder CharEntry (s.o.) zeigt
   auf den Beginn des zu seinem Char gehrigen Bitmusters.
   Jedes Bitmuster beginnt an einem ganzen Byte, aber innerhalb
   geht es je nach Zeichenbreite asynchron zu den Bytegrenzen */




/* Funktionen und Daten im GDI: */
extern const char  Helvetica_12[];
extern const char  Helvetica_Bold_12[];
extern const char  Fix5x7_7[];
extern const char  ArialBlack_16[];

extern void DrawLine         (POINT A, POINT B, int mode);
extern int  StringWidth      (char* S, const char* Font);
extern void CgFillRect       (RECT* R, int mode);
extern word CgPixel_at       (int X, int Y, int mode);
extern int  CgCh_at          (int X, int Y, char Ch, const char* Font, int mode);
extern int  CgStr_at         (int x, int y, char* P, const char* Font, int mode);

extern int  Get_dY           (const char* Font);
extern int  Get_Ascent       (const char* Font);
extern int  Get_Descent      (const char* Font);
extern int  Get_dX           (const char* Font, char Ch);


#endif






